/*
** Control logic for Goldilocks DAC
** www.myprius.co.za
** 2012, AJ BREDEKAMP
*/

#include <p33fj128gp802.h>

//OC1 ISR *NOT USED*
void __attribute__((__interrupt__, no_auto_psv)) _OC1Interrupt( void )
	{
		/* Interrupt Service Routine code goes here */
	IFS0bits.OC1IF = 0; // Clear OC1 interrupt flag
	}

//OC2 ISR *NOT USED*
void __attribute__((__interrupt__, no_auto_psv)) _OC2Interrupt( void )
	{
		/* Interrupt Service Routine code goes here */
	IFS0bits.OC1IF = 0; // Clear OC1 interrupt flag
	}

//Timer2 ISR *NOT USED*
void __attribute__((__interrupt__, no_auto_psv)) _T2Interrupt(void)
	{
		/* Interrupt Service Routine code goes here */
	IFS0bits.T2IF = 0; // Clear Timer3 Interrupt Flag
	}

//Timer3 ISR *NOT USED*
void __attribute__((__interrupt__, no_auto_psv)) _T3Interrupt(void)
	{
		/* Interrupt Service Routine code goes here */
	IFS0bits.T3IF = 0; // Clear Timer3 Interrupt Flag
   	}

int main()
{

    // Initializations----------------------------------------------------------------------------------------

    //Set Oscillator Configuration for 33.8688 MIPS
	OSCCON = 0x0300;			// 01100000000 	--> Primary oscillator (XT, HS, EC) with PLL
	CLKDIV = 0x0000;			// 
	PLLFBD = 0x001E;			// 11110 		--> /32  PLLDIV
	
    // set the priority levels
	_IC1IP = 4;					//DEFAULT

    // ---------------------------------------------------
    // ***************************************************
    // PERIPHERAL PINS------------------------------------
    // Configure Peripheral pin select registers
   
    // OC1 10010 RP8 tied to Output Compare 1, 			RP8R<4:0> - CLOX
    _RP8R0 = 0;
	_RP8R1 = 1;
 	_RP8R2 = 0;
	_RP8R3 = 0;
	_RP8R4 = 1;
    // OC2 10011 RP7 tied to Output Compare 2, 			RP7R<4:0> - STR1
    _RP7R0 = 1;
	_RP7R1 = 1;
 	_RP7R2 = 0;
	_RP7R3 = 0;
	_RP7R4 = 1;

    DAC1CONbits.DACEN = 0; 		// Disable DAC
    AD1CON1bits.ADON  = 0;  	// Disable ADC
   
    // ----------------------------------------------------
    // ****************************************************
    // OUTPUT COMPARE MODULES------------------------------
    // Setup Output Compare Module 1 for Toggle Mode -- OUTPUT = Fs x 384 = 16934400 Hz/4 = 4233600 Hz
    // Used to clock SAA7030 Digital Filter --> CLOX
    // Associated with Timer 2
	OC1CONbits.OCM = 0b000; 	// Disable Output Compare Module
	OC1CONbits.OCM = 0b010; 	// Define Initial State for OC1 Pin (High if OCM=0b010) 
	OC1CONbits.OCM = 0b000; 	// Disable Output Compare Module
	OC1CONbits.OCTSEL = 0; 		// Select Timer2 as output compare time base
	OC1R = 0; 					// Load the Compare Register Value 
	IPC0bits.OC1IP = 0x01; 		// Set Output Compare 1 Interrupt Priority Level
	IFS0bits.OC1IF = 0; 		// Clear Output Compare 1 Interrupt Flag 
	IEC0bits.OC1IE = 0; 		// Disable Output Compare 1 interrupt
	OC1CONbits.OCM = 0b011; 	// Select the Output Compare mode, Toggle Mode

	// Setup Output Compare Module 2 in Continuous Pulse mode
    // Used for SAA7030 STR1 strobe, frequency 44.1kHz. Rising edge follows completion of SAA7030 input data stream
    // Associated with Timer 3
	OC2CONbits.OCM = 0b000; 	// Disable Output Compare Module
	OC2CONbits.OCTSEL = 1; 		// Select Timer 3 as output compare time base
	OC2R = 1435; 				// Load the Compare Register Value for rising edge
	OC2RS = 1485; 				// Load the Compare Register Value for falling edge
	IPC1bits.OC2IP = 0x01; 		// Set Output Compare 2 Interrupt Priority Level
	IFS0bits.OC2IF = 0; 		// Clear Output Compare 2 Interrupt Flag  
	IEC0bits.OC2IE = 0; 		// Disable Output Compare 2 interrupt
	OC2CONbits.OCM = 0b101; 	// Select the Output Compare mode, Continuous Pulse mode
    // ----------------------------------------------------
    // ****************************************************
    // TIMERS----------------------------------------------
    // Initialize and enable Timer2
    // Clock Timer 2 from internal clock source Fosc
	T2CONbits.TON = 0; 			// Disable Timer
	T2CONbits.TCS = 0; 			// Select internal instruction cycle clock 
	T2CONbits.TGATE = 0; 		// Disable Gated Timer mode
	T2CONbits.TCKPS = 0b00; 	// Select 1:1 Prescaler
	TMR2 = 0x00; 				// Clear timer register
	PR2 = 7; 					// Load the period value, Timer is reset on period match 
                                // This value takes Fosc (33.8688MHz) and divide by 8 --> 4.2336MHz --> CLOX of SAA7030
	IPC1bits.T2IP = 0x01; 		// Set Timer2 Interrupt Priority Level
	IFS0bits.T2IF = 0; 			// Clear Timer2 Interrupt Flag
	IEC0bits.T2IE = 0; 			// Disable Timer2 interrupt

	// Initialize and enable Timer3
    // Clock Timer 3 from internal clock source Fosc
    T3CONbits.TON = 0; 			// Disable Timer
	T3CONbits.TCS = 0; 			// Select internal instruction cycle clock 
	T3CONbits.TGATE = 0; 		// Disable Gated Timer mode
	T3CONbits.TCKPS = 0b00; 	// Select 1:1 Prescaler
	TMR3 = 0x00; 				// Clear timer register
	PR3 = 1535; 				// Load the period value, Timer is reset on period match 
                                // This value takes Fosc (33.8688MHz) and divide by 768 --> 44.1kHz
	IPC2bits.T3IP = 0x01; 		// Set Timer3 Interrupt Priority Level
	IFS0bits.T3IF = 0; 			// Clear Timer3 Interrupt Flag
	IEC0bits.T3IE = 0; 			// Disable Timer3 interrupt
    
    //Activate Timers
    T2CONbits.TON = 1; 			// Start Timer2
	T3CONbits.TON = 1; 			// Start Timer3

    // Main Loop ---------------------------------------------------------------------------------------------   
	while(1)
	{   
   		//Nothing is done here. Everything is interrupt driven
	} 

	return 0;
	
} // main
//END --------------------------------------------------------------------------------------------------------
